<?php

use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Util\Toast\Toast;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Str;

if (function_exists('has_permission') === false) {
    function has_permission(string $permission): bool
    {
        $admin = optional(auth()->user())->admin;

        if ($admin === null) {
            return false;
        }

        try {
            return $admin->hasPermissionTo($permission);
        } catch (\Spatie\Permission\Exceptions\PermissionDoesNotExist $exception) {
            return false;
        }
    }
}

if (function_exists('toast') === false) {
    function toast(): Toast
    {
        return app('toast');
    }
}

if (function_exists('asOption') === false) {
    function asOption(string $option): array
    {
        return ['label' => Str::ucfirst(Str::snake($option, ' ')), 'value' => $option];
    }
}

if (function_exists('get_delegate_form_stage') === false) {
    function get_delegate_form_stage(Form $regForm): FormStep
    {
        if (Auth::user()->delegate->currentStep()->exists()) {
            return Auth::user()->delegate->currentStep;
        }

        return $regForm->steps->first();
    }
}
