<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Domain\Event\Form\Enums\FormType;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Support\Str;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Enum\Laravel\HasEnums;

class Form extends Model implements AuditableInterface
{
    use HasEnums, Auditable, HasTable;

    public const KEY = 'form';

    /** @var array<string,mixed> */
    protected $enums = [
        'type' => FormType::class.':nullable',
    ];

    /** @var array<string> */
    protected $with = [
        'steps.sections.questions.options',
        'steps.sections.questions.validations',
    ];

    /** @var array<string> */
    protected $fillable = [
        'name',
        'type',
        'uuid',
        'event_id',
    ];

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function steps(): HasMany
    {
        return $this->hasMany(FormStep::class)->orderBy('order');
    }

    public function finalStep(): HasOne
    {
        return $this->hasOne(FormStep::class)->orderBy('order', 'desc');
    }

    public function scopeForEvent(Builder $query, Event $event): Builder
    {
        return $query->where('event_id', $event->id);
    }

    public function scopeRegistration(Builder $query): Builder
    {
        return $query->where('type', FormType::Registration());
    }

    public static function typesAsOptions(): array
    {
        $enum = collect(array_flip(FormType::toArray()));

        return $enum
            ->map(static function ($value): array {
                return ['label' => Str::ucfirst(Str::snake($value, ' ')), 'value' => $value];
            })
            ->toArray();
    }
}
