<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Domain\Model;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class QuestionOption extends Model implements AuditableInterface
{
    use Auditable;

    const KEY = 'question_options';

    /** @var array<string> */
    protected $fillable = [
        'name',
        'order',
        'question_id',
        'uuid',
    ];

    public function question(): BelongsTo
    {
        return $this->belongsTo(Question::class);
    }

    public function scopeForQuestion(Builder $query, Question $question): Builder
    {
        return $query->where('question_id', '=', $question->id);
    }

    public function toArray(): array
    {
        return [
            'id' => $this->id,
            'label' => $this->name,
            'value' => $this->name,
        ];
    }
}
