<?php

namespace EventPortal\Http\Controllers\Delegate\Event;

use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Event\Form\Handlers\SaveForm;
use EventPortal\Domain\Event\Form\Models\FormData;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\ValueObjects\EventDetails;
use EventPortal\Domain\EventDelegate\Handlers\RegisterDelegate;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Util\DelegateRender;
use EventPortal\Enums\InertiaMethod;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class RegistrationController
{
    public function index(Event $event): Response
    {
        $regForm = $event->forms()->registration()->first();
        $step = get_delegate_form_stage($regForm);

        $form = Form::makeFromDatabase($regForm, $step)
            ->model(new FormData)
            ->context($event)
            ->addHandler(new SaveForm)
            ->addActions(static function (Event $event): array {
                return [
                    Button::make('Back')
                        ->variant('secondary')
                        ->method(InertiaMethod::VISIT())
                        ->route('register.index', ['event' => $event->slug]),
                    Button::make('Save for later')
                        ->variant('secondary')
                        ->method(InertiaMethod::VISIT())
                        ->route('register.update', ['event' => $event->slug]),
                ];
            })
            ->addPrimaryAction(static function (Event $event): Button {
                return Button::make('Next step')
                    ->method(InertiaMethod::POST())
                    ->route('register.store', ['event' => $event->slug]);
            })
            ->render();

        //TODO: create delegate registration component
        return DelegateRender::component('Delegate/Contact/Index')
            ->key('contact.index')
            ->withForm($form)
            ->withEventDetails(EventDetails::make($event))
            ->get();
    }

    public function store(Request $request, Event $event): RedirectResponse
    {
        $regForm = $event->forms()->registration()->first();
        $step = get_delegate_form_stage($regForm);

        Form::makeFromDatabase($regForm)
            ->model(new FormData)
            ->context($step, $event, $regForm)
            ->addHandler(new SaveForm)
            ->addHandler(new RegisterDelegate)
            ->validate($request->input())
            ->handle($request->input());

        if ($regForm->finalStep->is($step)) {
            return redirect()->route('delegate.landing', ['event' => $event->slug]);
        }

        return redirect()->route('register.index', ['event' => $event->slug]);
    }
}
